/*************************************************************************************/
/* */
/*************************************************************************************/

#define AT91M		1
#define AT91_PTRS	1
#include "at91xx.h"


/*PB26*/
#define LED4	(1<<26)
/*PA27*/
#define LED3	(1<<27)
/*PA28*/
#define LED2	(1<<28)
/*PA29*/
#define LED1	(1<<29)

/*PA9*/
#define KEY_S1  (1<<9)
/*PB16*/
#define KEY_S2  (1<<16)
/*PB19*/
#define KEY_S3  (1<<19)
/*PB20*/
#define KEY_S4  (1<<20)

#define PortBLeds 0x0000ff00

#define S1 1
#define S2 S1+1
#define S3 S2+1
#define S4 S3+1

#define WAIT 0x10000


/*************************************************************************************/
/* Function prototypes */

void loop();
void leds_on(StructPIO *port, int leds);
void leds_off(StructPIO *port, int leds);
int  readKeys();
void wait(void);

/*************************************************************************************/

StructPIO *at91_pio;

extern int main ( void ) 
{
	at91_pio=PIOA_BASE;

	at91_pio->PIO_ODR= KEY_S1; 		/* Defines the PIOs as input */
	at91_pio->PIO_IDR  = 0xFFFFFFFF;	/* Disable PIO interrupts */
	at91_pio->PIO_IFDR= LED2 | LED3 | LED1; /* Disable the filter on PIOs */	
	at91_pio->PIO_PER  = LED2 | LED3 | LED1 | KEY_S1;	/*enable pio-pins */
	at91_pio->PIO_OER = LED2 | LED3 | LED1;

	at91_pio=PIOB_BASE;
	at91_pio->PIO_ODR= KEY_S2 | KEY_S3 | KEY_S4;
	at91_pio->PIO_IDR  = 0xFFFFFFFF;	
	at91_pio->PIO_IFDR= LED4 | PortBLeds;
	at91_pio->PIO_PER = LED4 |KEY_S2 | KEY_S3 | KEY_S4 | PortBLeds;

	at91_pio->PIO_OER = LED4 | PortBLeds;

	PMC_BASE->PMC_PCER|=PMC_PIOA|PMC_PIOB;

	leds_off(PIOB_BASE, PortBLeds);	/*clear 8 Leds PB8..15*/

	loop();

	return(0);
}

/*************************************************************************************/
/* sign per LEDs */
void loop()
{
	int volatile key;

	while(1)
	{
		key=readKeys();
		switch(key)
		{
		  case S1:
			leds_off(PIOB_BASE, LED4);
			leds_off(PIOA_BASE, LED2|LED3);
			leds_on(PIOA_BASE, LED1);
			break;
		  case S2:
			leds_off(PIOB_BASE, LED4);
			leds_off(PIOA_BASE, LED3|LED1);
			leds_on(PIOA_BASE, LED2);
			break;
		  case S3:
			leds_off(PIOB_BASE, LED4);
			leds_off(PIOA_BASE, LED2|LED1);
			leds_on(PIOA_BASE, LED3);
			break;
		  case S4:
			leds_off(PIOA_BASE, LED2|LED3|LED1);
			leds_on(PIOB_BASE, LED4);
			break;
		 default:
			leds_off(PIOB_BASE, LED4);
			leds_off(PIOA_BASE, LED2|LED3|LED1);
		};
	}
}

/*************************************************************************************/

void leds_on(StructPIO *port, int leds)
{
	at91_pio=port; 
	at91_pio->PIO_SODR = leds;
}

/*************************************************************************************/

void leds_off(StructPIO *port, int leds)
{
	at91_pio=port; 
	at91_pio->PIO_CODR = leds;
}

/*************************************************************************************/
int  readKeys()
{
	int volatile val=0;
	at91_pio=PIOA_BASE;
	val=at91_pio->PIO_PDSR;
	if (val&KEY_S1) return S1;

	at91_pio=PIOB_BASE;
	val=at91_pio->PIO_PDSR;
	if (val&KEY_S2) return S2;
	if (val&KEY_S3) return S3;
	if (val&KEY_S4) return S4;
	return 0;
}

/*************************************************************************************/
void wait(void)
{
	int n;
	for(n=0;n<WAIT;n++) {n++;n--;}
}

/*************************************************************************************/
void __gccmain()
{
}
