.equ STACK_LIMIT,     0xC00	/* 3K */
.equ IRQ_STACK_SIZE,  (4*4)

/* for Mem-Ctrl */
.equ	EBI_BASE,0xFFE00000

/* Adress and CFG NSC0 (Flash) */
.equ EBI_CSR0 ,0x01002529

/* Adress and CFG NSC1 (SRAM) */
.equ EBI_CSR1 ,0x02002121

/* for LCD or PIO_8255 on CS3*/
.equ EBI_CSR3 ,0x03002426


/* Power Management Controller Base Address */
.equ APMC_BASE  ,     0xFFFF4000

.equ APMC_CGMR  ,       0x20       /* - Clock Generator Mode Register */
.equ APMC_SR    ,       0x30       /* - Status Register */
	.equ APMC_MOSCS     ,     0x1     /*- Main Osillator Status */
	.equ APMC_PLL_LOCK  ,     0x2     /*- PLL Lock Status */



/* ##################################################################### */

	.global	__main
__main:	
	b resetvec
	b undefined_instruction
	b software_interrupt
	b prefetch_abort
	b data_abort
	b not_used
	b irq
	b fiq

/* ##################################################################### */

undefined_instruction:
	b undefined_instruction
software_interrupt:
	b software_interrupt
prefetch_abort:
	b prefetch_abort
data_abort:
	b data_abort
not_used:
	b not_used
irq:
	b irq
fiq:
	b fiq
/* ##################################################################### */

.global resetvec
resetvec:
	mov	sp,#0x400

low_level_init:

/* - Speed up the System Frequency 
   - from 32KHz to 16MHz and last upto 32MHz
BITs for #APMC_CGMR:
--pppppp	PllCount (0)
oooooooo	OsCount (2F)
ccmmmmmm	CSS ('00' RTC / '01' Main osz. / '10' phase lock loop)   MUL (000000 --> PLL deactivated)
-sca-deb	- Pre[sca]ler (000) - MCKO[D]S (0) MOSC[E]N (1)  MOSK[B]YP (0=crystal 1 -->xin)
*/

                ldr     r0, =0x002F0002             /* MOSCEN = 1, OSCOUNT = 47  (1.4ms/30s) */
                ldr     r1, =APMC_BASE              /* Get the APMC Base Address */
                str     r0, [r1, #APMC_CGMR]        /* Store the configuration of the Clock Generator */

/* - Reading the APMC Status register to detect when the oscillator is stabilized */
                mov     r4, #APMC_MOSCS
MoscsLoop:
                ldr     r2, [r1, #APMC_SR]
                and     r2, r4,r2
                cmp     r2, #APMC_MOSCS
                bne     MoscsLoop

/* - Commuting from Slow Clock to Main Oscillator (16Mhz) */
                ldr     r0, =0x002F4002             /* use 16MHz on X-Pins, MUL ==> 16MHz*1 */
                str     r0, [r1, #APMC_CGMR]        /* Store the configuration of the Clock Generator */

/* -Setup the PLL */
                ldr     r0, =0x032F4102             /* PLLCOUNT = 3, use 16MHz on X-Pins, MUL ==> 16MHz*2 */
                str     r0, [r1, #APMC_CGMR]        /* Store the configuration of the Clock Generator */

/* - Reading the APMC Status register to detect when the PLL is stabilized */
                mov     r4, #APMC_PLL_LOCK
Pll_Loop:
                ldr     r2, [r1, #APMC_SR]
                and     r2, r4,r2
                cmp     r2, #APMC_PLL_LOCK
                bne     Pll_Loop

/* - Commuting from 16Mhz to PLL @ 32MHz */
                ldr     r0, =0x032F8102             /* phase lock loop, MUL = 1 */
                str     r0, [r1, #APMC_CGMR]        /* Store the configuration of the Clock Generator */

/* - Now the Master clock is the output of PLL @ 32MHz */


/* Remapping now */
	ldr	r10,=InitTableEBI		/* get the address of the chip select register image */
	movs	r0, pc, LSR #20		/* pc > 0x100000 */
	moveq	r10, r10, LSL #12	/* Mask the 12 highest bits of the address */
	moveq	r10, r10, LSR #12
	
	/* Copy Chip Select Register Image to Memory Controller and command remap */
	ldmia	r10!, {r0-r9,r11-r12}	/* load the complete image and the EBI base */
	stmia	r11!, {r0-r9}			/* store the complete image with the remap command */
	mov	pc, r12						/* jump and break the pipeline */


/* ##################################################################### */
AfterReMap:

	b	AfterReMap

/* ##################################################################### */
InitTableEBI:
	.long	EBI_CSR0		/* 0x01000000, 16MB, 0 hold, 16 bits, 8 WS */
	.long	EBI_CSR1		/* 0x02000000, 16MB, 0 hold, 16 bits, 1 WS, Byte Select access */
	.long	0x20000000		/* unused */
	.long	EBI_CSR3		/* CS3*/
	.long	0x40000000		/* unused */
	.long	0x50000000		/* unused */
	.long	0x60000000		/* unused */
	.long	0x70000000		/* unused */
	.long	0x00000001		/* REMAP command */
	.long	0x00000006		/* 7 memory regions, standard read */
	.long	EBI_BASE		/* EBI Base address */
	.long 	AfterReMap
